﻿using Http.Core;
using Http.Core.Contexts;
using Http.Core.Exceptions;
using Http.Core.Handlers;
using Microsoft.Extensions.DependencyInjection;
using PCUT.Pages;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.ApplicationModel;
using Windows.ApplicationModel.Activation;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Core;
using Windows.UI.Popups;
using Windows.UI.ViewManagement;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using static Http.Core.Constants.HttpConstants;

namespace PCUT
{
    /// <summary>
    /// Provides application-specific behavior to supplement the default Application class.
    /// </summary>
    sealed partial class App : Application
    {
        /// <summary>
        /// Initializes the singleton application object.  This is the first line of authored code
        /// executed, and as such is the logical equivalent of main() or WinMain().
        /// </summary>
        public static NavigationTarget NextPage { get; set; } = NavigationTarget.None;
        public enum NavigationTarget
        {
            None,
            DesignCenter,
            DataCenter,
            AdminCenter
        }
        public App()
        {
            this.InitializeComponent();
            this.Suspending += OnSuspending;
            this.UnhandledException += HandleException;
            AppContainer = ConfigDI();
            HttpClientFactory
                .AddHttpClient(ClientNames.AuthClient)
                .AddConfigureClient(client => { client.BaseAddress = new Uri(Auth.BaseUrl); })
                .AddHttpMessageHandler(() => new DeviceAuthHandler());
            HttpClientFactory
                .AddHttpClient(ClientNames.ApiClient)
                .AddConfigureClient(client => { client.BaseAddress = new Uri(Api.BaseUrl); })
                .AddHttpMessageHandler(() => new DeviceAuthHandler())
                .AddHttpMessageHandler(() => new AuthenticationHandler());
            UserContext.Instance.DebugFeaturesEnabled = false;
        }
        public IServiceProvider AppContainer { get; }

        /// <summary>
        /// Invoked when the application is launched normally by the end user.  Other entry points
        /// will be used such as when the application is launched to open a specific file.
        /// </summary>
        /// <param name="e">Details about the launch request and process.</param>
        protected override void OnLaunched(LaunchActivatedEventArgs e)
        {
            UserContext.Instance.SetAppVersion(GetAppVersion());
            ApplicationView.GetForCurrentView().IsScreenCaptureEnabled = UserContext.Instance.DebugFeaturesEnabled;
            Frame rootFrame = Window.Current.Content as Frame;
            // Do not repeat app initialization when the Window already has content,
            // just ensure that the window is active
            if (rootFrame == null)
            {
                // Create a Frame to act as the navigation context and navigate to the first page
                rootFrame = new Frame();

                rootFrame.NavigationFailed += OnNavigationFailed;

                if (e.PreviousExecutionState == ApplicationExecutionState.Terminated)
                {
                    //TODO: Load state from previously suspended application
                }

                // Place the frame in the current Window
                Window.Current.Content = rootFrame;
            }

            if (e.PrelaunchActivated == false)
            {
                if (rootFrame.Content == null)
                {
                    // When the navigation stack isn't restored navigate to the first page,
                    // configuring the new page by passing required information as a navigation
                    // parameter
                    rootFrame.Navigate(typeof(LoginPage), e.Arguments);
                }
                // Ensure the current window is active
                Window.Current.Activate();
            }
        }


        /// <summary>
        /// Invoked when Navigation to a certain page fails
        /// </summary>
        /// <param name="sender">The Frame which failed navigation</param>
        /// <param name="e">Details about the navigation failure</param>
        void OnNavigationFailed(object sender, NavigationFailedEventArgs e)
        {
            throw new Exception("Failed to load Page " + e.SourcePageType.FullName);
        }

        /// <summary>
        /// Invoked when application execution is being suspended.  Application state is saved
        /// without knowing whether the application will be terminated or resumed with the contents
        /// of memory still intact.
        /// </summary>
        /// <param name="sender">The source of the suspend request.</param>
        /// <param name="e">Details about the suspend request.</param>
        private void OnSuspending(object sender, SuspendingEventArgs e)
        {
            var deferral = e.SuspendingOperation.GetDeferral();
            //TODO: Save application state and stop any background activity
            deferral.Complete();
        }

        private async void HandleException(object sender, Windows.UI.Xaml.UnhandledExceptionEventArgs e)
        {
            if (e.Exception is AppOutdatedException outdatedException)
            {
                e.Handled = true;
                var dialog = new MessageDialog(outdatedException.Message);
                dialog.Commands.Add(new UICommand("Download", async (command) =>
                {
                    var _ = await Windows.System.Launcher.LaunchUriAsync(
                        new Uri("https://pcut.vn/download"),
                        new Windows.System.LauncherOptions { TreatAsUntrusted = false });
                }));
                dialog.Commands.Add(new UICommand("Close"));
                dialog.DefaultCommandIndex = 0;
                dialog.CancelCommandIndex = 1;
                await dialog.ShowAsync();
            }
        }

        IServiceProvider ConfigDI()
        {
            var serviceCollection = new ServiceCollection();
            return serviceCollection.BuildServiceProvider();
        }

        private string GetAppVersion()
        {
            var packageVersion = Package.Current.Id.Version;
            return $"{packageVersion.Major}.{packageVersion.Minor}.{packageVersion.Build}.{packageVersion.Revision}";
        }
    }
}
